function [E,RES,stat] = SCV(Il,Ir,TC,SIFTr, model, fp,fn);
%SCV - Sequential Correspondence Verification
% 
% [L,RES,stat] = SCV(Il,Ir,TC,SIFTr, model, alpha,beta);
%
% input:
% ------
%   Il,Ir - left and right images (gray scale)
%   TC - tentative correspondences to be verified (18xn)
%        (per column:
%            correspondence of local affine frames, i.e. 3 to 3 points in
%            homogeneous coordinates:
%               [p1,p1',p2,p2',p3,p3'], where  pi = [pi_x,pi_y,1] )
%       
%   SIFTr - ratio of first to second closest SIFT descriptor of tentative 
%           correspondences (1xn)
%
%   model - trained model (distributed with the toolbox)
%        .P - pdfs  
%        .SVM - SVM models
%
%   alpha, beta - Wald's SPRT tes parameters (false positive, negative)
%
%
% output:
% -------
%  L - resulting quality of correspondences (1xn), 
%      (likelihood ratio of a sequential classifier)
%
%  RES - results (9xn)
%            [1) \bar g ............................. ]
%            [2) \bar c ............................. ]
%            [3) \bar u ............................. ]
%            [4) SIFT-ratio ......................... ]
%            [5) number of correlations ............. ]
%            [6) stage when decided ................. ] 
%            [7) SVM output (q) ..................... ]
%            [8) Wald SPRT test decision (1/0/-1) ... ] 
%            [9) queu exhausted (1/0) ............... ]
%
%  stat - statistics of the sequential decision process
%     .proc  - percentage of decided correspondences in subsequent stages
%     .corr  - number of correspondences in subsequent stages
%     .total_rel_corr - relative number of correlations per correspondence
%     .time  - timing of the procedure
%
%
% The algorithm is described in:
%   Jan Cech, Jiri Matas, Michal Perdoch; Efficient Sequential
%   Correspondence Selection by Cosegmentation; In Proc. CVPR, 2008.
%  
% Jan Cech, cechj@cmp.felk.cvut.cz, Apr 30, 2009.
%

if ndims(Il)>2, error('Gray-scale intensity images expected.'); end

time = [];

t0 = cputime;
IN = build_seq_input_matrix(model, fp, fn);
time.init = cputime-t0;

%TC (18xn ~ 3xLAFs to SEEDs) -----------------------------------------
A = laf2H(TC);
%SEEDs = [round(h2e(TC(1:3,:))'),A';     %all 3 points
%         round(h2e(TC(7:9,:))'),A';
%         round(h2e(TC(13:15,:))'),A'];
if size(TC,2)==3 | size(TC,2)==4, %bug v h2e
    SEEDs = [[round(TC(1:2,:));round(TC(7:8,:));round(TC(13:14,:))]',A'];
else
    SEEDs = [[round(h2e(TC(1:3,:)));round(h2e(TC(7:9,:)));round(h2e(TC(13:15,:)))]',A'];
end
SEEDs(:,end+1) = SIFTr';

%fixed parameters of the growing process ----------------------------
w = 2;
local_search = [-1,-1; -1,0; -1,1; 0,-1; 0,0; 0,1; 1,-1; 1,0; 1,1];
c_thr = 0.5;
searchrange = [-5,5];
searchrangeV = [-5,5];
if isfield(model,'epsilon');
    epsilon = model.epsilon;
else
    epsilon = 0.01;
end
i_seed_accept = 0; 
maxK = 2;

t0 = cputime;
[D,Dv,W,K,I,Q,OUT]=dgrow4e_bf_mex(Il,Ir,SEEDs, ...
                              w,local_search,c_thr,searchrange, ...
                              searchrangeV,epsilon,i_seed_accept,maxK, ...
                              IN);
time.growth = cputime-t0;                          
                          
RES = OUT';

%likelihood-ratio from SVM-output evaluation -------------------------

t0 = cputime;

stat = [];
stat.proc = repmat(NaN,1,4);
stat.corr = repmat(NaN,1,4);
stat.total_rel_corr = NaN;

num_corresp = size(RES,2);

E = repmat(NaN, 1, size(RES,2));
for ds=1:length(model.steps), %all decision stages
    i = (RES(6,:)==ds);
    if ds==1,
        pp = model.P{1}.sift_ratio.pp;
        xp = model.P{1}.sift_ratio.xp;
        pn = model.P{1}.sift_ratio.pn;
        xn = model.P{1}.sift_ratio.xn;
    else
        pp = model.P{ds}.data123sift.p;
        xp = model.P{ds}.data123sift.xp;
        pn = model.P{ds}.data123sift.n;
        xn = model.P{ds}.data123sift.xn;
    end
    E(i) = get_lratio(RES(7,i), pp,xp,pn,xn); %likelihood ratio
    
    %statistics
    stat.proc(ds) = sum(i)/num_corresp * 100;
    stat.corr(ds) = sum(RES(5,i));
end

time.likelihoods = cputime-t0;

stat.total_rel_corr = sum(RES(5,:))/num_corresp;
stat.time = time;

