function [M] = build_seq_input_matrix(model, alpha, beta)
%build_seq_input_matrix - constructs an input matrix for a sequential
%                         classifier
%
%  [M] = build_seq_input_matrix(model, alpha, beta)
%
%   model - P,SVM models (see P-SVM_p_#.mat)
%   alpha, beta - false positive, false negative parameters
%   
%   M(8x num_of_stages) - matrix which is passed to a sequential classifier
%                         in C
%      
%        -> stage (i)
%    i1 [..................................................] decision
%    i2 [..................................................]   intervals
% steps [0.................................................] num of steps
%    w1 [..................................................] SVM weights
%    w2 [..................................................] .
%    w3 [..................................................] .
%    w4 [..................................................] .
%     b [..................................................] SVM bias
%
% 
%  decision intervals
%      t > i1     positive
%      t < i2     negative
%   i2 < t < i1   test not conclusive
%

M = zeros(8,length(model.steps));

for i=1:length(model.steps)
    if i==1,
        xp = model.P{i}.sift_ratio.xp;  %sift_ratio only
        xn = model.P{i}.sift_ratio.xn;
        w = [0,0,0,-1]'; 
        b = 0;
    else
        xp = model.P{i}.data123sift.xp;
        xn = model.P{i}.data123sift.xn;        
        w = model.SVM{i}.model_123sift.W;
        b = model.SVM{i}.model_123sift.b;
    end
    [i1,i2] = decision_intervals(xp',xn',alpha,beta);
    M(1,i) = i1;
    M(2,i) = i2;
    M(4:7,i) = w;
    M(8,i) = b;
end
M(3,:) = model.steps;

%plot(M(3,:),M(4:8,:),'+-'); legend('g','c','u','s_r','bias'); grid on


